#ifndef RAWCONFIGURATIONBLOCKV0_H
#define RAWCONFIGURATIONBLOCKV0_H
#include "DAQDataFormats/RawConfigurationHeader.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawConfigurationBlock
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE configurationblock
#include "DAQDataFormats/DataFormatLast.inc"

// forward declarations
namespace daqdataformats {
    class RawConfigurationTail;
    class RawConfigurationSystemID;
}

namespace daqdataformats {

    namespace CURRENT_FIELDS_SUB_NAMESPACE {
        static const uint32_t ConfigurationBlockMaximumSize = 10000;
    }

namespace  VERSION_NAMESPACE {
    namespace CURRENT_FIELDS_SUB_NAMESPACE {
        static const version_t Version_ConfigurationSystemID = 0;
        static const version_t Version_ConfigurationTail     = 0;
    }// end of namespace configurationblock

    
  class RawConfigurationBlock : public PREVIOUS_VERSION_CLASS_NAME {
    
        public:
            RawConfigurationBlock (const version_t); ///< The default constructor
            RawConfigurationBlock (const RawConfigurationBlock&);
            ~RawConfigurationBlock(); ///< Destructor

            void     init            ();
            using    PREVIOUS_VERSION_CLASS_NAME::print;
            void     print           (std::ostream& os=std::cout) const;
            uint32_t sizeofdata      () const { return _header->getConfigurationBlockSize(); }
            bool     setBufferSource (const void* pointer);

            using    PREVIOUS_VERSION_CLASS_NAME::readData;	// unhide the reset of the readData methods
            void*    readData(const void*);

            INIT_FIRST(close                            ,      , bool, const bool)
	    INIT_FIRST(getHeader                        ,      , daqdataformats::RawConfigurationHeader*, )
	    INIT_FIRST(getTail                          ,      , daqdataformats::RawConfigurationTail*, )
	    INIT_FIRST(getTailConst                     , const, daqdataformats::RawConfigurationTail*, )
	    INIT_FIRST(getSystemID                      ,      , daqdataformats::RawConfigurationSystemID*, )
	    INIT_FIRST(getSystemIDConst                 , const, daqdataformats::RawConfigurationSystemID*, )
            INIT_FIRST(getSystemIDWithInput             ,      , daqdataformats::RawConfigurationSystemID*, const uint32_t)
            INIT_FIRST(getSystemIDConstWithInput        , const, daqdataformats::RawConfigurationSystemID*, const uint32_t)
            INIT_FIRST(setFloatingSystemIDToBufferSource, const, bool, const void*)
            INIT_FIRST(setFloatingSystemID              , const, bool, const uint32_t)
            INIT_FIRST(addSystemID                      ,      , bool, const void*)

	    INIT_FIRST(getMarkerHi1             , const, uint32_t, )
	    INIT_FIRST(getMarkerHi2             , const, uint32_t, )
	    INIT_FIRST(getMarkerLo1             , const, uint32_t, )
	    INIT_FIRST(getMarkerLo2             , const, uint32_t, )
	    INIT_FIRST(getConfigurationVersion  , const, uint32_t, )
	    INIT_FIRST(getConfigurationBlockSize, const, uint32_t, )
	    INIT_FIRST(getSystemNumber          , const, uint32_t, )

            INIT_FIRST_GET_AND_SET(DetectorID       , uint32_t)
            INIT_FIRST_GET_AND_SET(ResourceManagerID, uint32_t)
            INIT_FIRST_GET_AND_SET(RunNumber        , uint32_t)
            INIT_FIRST_GET_AND_SET(RunType          , uint32_t)
            INIT_FIRST_GET_AND_SET(Partition        , uint32_t)
            INIT_FIRST_GET_AND_SET(Subrun           , uint32_t)
            INIT_FIRST_GET_AND_SET(SystemVersionIDX , uint32_t)

        protected:

            /// Setting versions
            bool setVersions() {
                _version_ConfigurationSystemID = CURRENT_FIELDS_SUB_NAMESPACE::Version_ConfigurationSystemID;
                _version_ConfigurationTail     = CURRENT_FIELDS_SUB_NAMESPACE::Version_ConfigurationTail;
                return true;
            }
            
            uint32_t getPredefinedSize() const {return 1;}

        protected:
            mutable daqdataformats::RawConfigurationTail*      _tail;
            daqdataformats::RawConfigurationHeader*            _header;                //<! Header. Embedded into the InternalBuffer of the RawConfigurationBlockV0
            mutable daqdataformats::RawConfigurationSystemID*  _floatingSystemID;      //<! Configuration system ID
            mutable int                                        _floatingSystemIDNumber;//<! Floating system ID Number

            version_t                                          _version_ConfigurationSystemID;
            version_t                                          _version_ConfigurationTail;

  };
  
  
}} // END NAMESPCE daqdataformats
#endif /* RAWCONFIGURATIONBLOCKV0_H */
