#ifndef RAWCONFIGURATIONNAMEV0_H
#define RAWCONFIGURATIONNAMEV0_H
#include "DAQDataFormats/RawDAQData.h"
#include "DAQDataFormats/Macros.h"
#include "DAQDataFormats/BitFields.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawConfigurationName
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE rawconfname
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

namespace  VERSION_NAMESPACE {
    namespace CURRENT_FIELDS_SUB_NAMESPACE {

            enum ConfNameMASKS{
                MASK_Version = 0x00000FFF, //! Version number for the RawConfigurationNameV0 class (first of them) (12 bits)
                MASK_Size    = 0xFFFFF000, //! Size of the Name in words
                MASK_Char1   = 0xFF000000, //! Character 1 (8 bits)
                MASK_Char2   = 0x00FF0000, //! Character 1 (8 bits)
                MASK_Char3   = 0x0000FF00, //! Character 1 (8 bits)
                MASK_Char4   = 0x000000FF  //! Character 1 (8 bits)
            };

            enum ConfNameWORDS{
                WORD_Version = 0, //! Version number for the RawConfigurationNameV0 class (first of them) (12 bits)
                WORD_Size    = 0, //! Size of the Name in words
                WORD_Char    = 1  //! Character 1 (8 bits)
            };

            DEFINE_SHIFT_FROM_MASK(Version)
            DEFINE_SHIFT_FROM_MASK(Size)
            DEFINE_SHIFT_FROM_MASK(Char1)
            DEFINE_SHIFT_FROM_MASK(Char2)
            DEFINE_SHIFT_FROM_MASK(Char3)
            DEFINE_SHIFT_FROM_MASK(Char4)

    }// end of namespace rawconfhdr



    class RawConfigurationName : public PREVIOUS_VERSION_CLASS_NAME {
      
        public:
            /// Constructors
            RawConfigurationName (const version_t); ///< The default constructor
            RawConfigurationName (const RawConfigurationName&);
            ~RawConfigurationName(); ///< Destructor

            void     init            ();
            using    PREVIOUS_VERSION_CLASS_NAME::print;
            void     print           (std::ostream& os=std::cout) const;
            uint32_t sizeofdata      () const;
            bool     setBufferSource(const void* pointer);

            using    PREVIOUS_VERSION_CLASS_NAME::readData;	// unhide the reset of the readData methods
            void*    readData(const void*);
            
            IMPLEMENT_FIRST_GET_AND_SET(Version, uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(Size   , uint32_t)

	    INIT_FIRST(getHeaderSize , const, uint32_t, );

            INIT_FIRST(getName , const, uint32_t, std::string&);
            INIT_FIRST(setName ,      , uint32_t, const std::string&);

            INIT_FIRST(getChar1, const, uint32_t, const uint32_t);
            INIT_FIRST(setChar1,      , bool    , const uint32_t, const uint32_t);
            INIT_FIRST(getChar2, const, uint32_t, const uint32_t);
            INIT_FIRST(setChar2,      , bool    , const uint32_t, const uint32_t);
            INIT_FIRST(getChar3, const, uint32_t, const uint32_t);
            INIT_FIRST(setChar3,      , bool    , const uint32_t, const uint32_t);
            INIT_FIRST(getChar4, const, uint32_t, const uint32_t);
            INIT_FIRST(setChar4,      , bool    , const uint32_t, const uint32_t);

        protected:
            uint32_t getPredefinedSize() const {return CURRENT_FIELDS_SUB_NAMESPACE::WORD_Char;}


    };// end of class definition
}} // END NAMESPCE daqdataformats
#endif /* RAWCONFIGURATIONNAMEV0_H */
