#ifndef RAWDATABLOCKK_H
#define RAWDATABLOCKK_H
#include "DAQDataFormats/RawDataBlockV2.h"
#include "DAQDataFormats/RawMicroBlock.h"
#include "DAQDataFormats/Macros.h"

#include <stdarg.h>		// va_start

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawDataBlock
#define  LAST_CLASS_VERSION           2
#define  CURRENT_FIELDS_SUB_NAMESPACE datablock
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

    static const uint32_t DEFAULT_DATABLOCK_SIZE = 262144; /* Maximum MicroBlock Buffer Size 64k */

    class RawDataBlock : public PREVIOUS_VERSION_CLASS_NAME {
      
        public:
      
            /// Constructors
            /// I had to put version at the end, so that it would be compatible with the old BufferNodeEVB and DataLogger codes
      RawDataBlock (const uint32_t buffersize = DEFAULT_DATABLOCK_SIZE, const uint32_t tlvl=0, const version_t version=datablockheader::RAWDATABLOCKHEADER_LATEST_VERSION);                        //! The default constructor
            RawDataBlock (const RawDataBlock& copy_in);  //! Deep copy constructor
            RawDataBlock (const void* datablock_buffer, const version_t version = rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION, const uint32_t buffersize = DEFAULT_DATABLOCK_SIZE);       //! Shallow copy constructor
            /// Destructor
            ~RawDataBlock();

            /// Add the MicroBlock to _Buffer.
            /// If check_mc_flag = true, the method checks whether the microslice
            /// in the microblock is MC generatedand adds an appropriate flag to the DataBlock Header.
            bool addMicroBlock(const void* microblock_buffer,
                               const bool  check_mc_flag=true
                              )
            {RETURN_FUNCTION(addMicroBlock1)(microblock_buffer, check_mc_flag);}
            /// Add the MicroBlock of a known size to _Buffer.
            /// If check_mc_flag = true, the method checks whether the microslice
            /// in the microblock is MC generatedand adds an appropriate flag to the DataBlock Header.
            bool addMicroBlock(const void*    microblock_buffer,
                               const uint32_t microblock_size,
                               const bool     check_mc_flag=true
                              )
            {RETURN_FUNCTION(addMicroBlock2)(microblock_buffer, microblock_size, check_mc_flag);}
            /// Add the MicroBlock to _Buffer by passing the references of
            /// the MilliSlice and the MicroSlice. MilliSlice is the one that contains the MicroSlice.
            /// If check_mc_flag = true, the method checks whether the microslice
            /// in the microblock is MC generatedand adds an appropriate flag to the DataBlock Header.
            bool addMicroBlock(const void*     millislice_buffer,
                               const void*     microslice_buffer,
                               const version_t microblock_version=microblock::RAWMICROBLOCK_LATEST_VERSION,
                               const bool      check_mc_flag=true
                              )
            {RETURN_FUNCTION(addMicroBlock3)(millislice_buffer,microslice_buffer, microblock_version, check_mc_flag);}
            /// Add the MicroBlock to _Buffer by passing the references of
            /// the MilliSlice and the MicroSlice of a known size.
            /// MilliSlice is the one that contains the MicroSlice.
            /// If check_mc_flag = true, the method checks whether the microslice
            /// in the microblock is MC generatedand adds an appropriate flag to the DataBlock Header.
            bool addMicroBlock(const void*     millislice_buffer,
                               const void*     microslice_buffer,
                               const uint32_t  microslice_size,
                               const version_t microblock_version=microblock::RAWMICROBLOCK_LATEST_VERSION,
                               const bool      check_mc_flag = true
                              )
            {RETURN_FUNCTION(addMicroBlock4)(millislice_buffer, microslice_buffer, microslice_size, microblock_version, check_mc_flag);}

	    bool addDCMData(const void* dcmdata_buffer) {RETURN_FUNCTION(addDCMData1)(dcmdata_buffer);}
	    bool addDCMData(const void* dcmdata_buffer, const uint32_t dcmdata_size){RETURN_FUNCTION(addDCMData2)(dcmdata_buffer,dcmdata_size);}

            bool close(bool produce_crc=false) {RETURN_FUNCTION(close)(produce_crc);}

            IMPLEMENT_MAIN_STANDARD
            IMPLEMENT_MAIN_setBufferSource
            readDataStatus readData(const int fd){RETURN_FUNCTION(readDataFromFileDescriptor)(fd);}


            IMPLEMENT_GET_MAIN(Version, uint32_t);
            IMPLEMENT_GET_MAIN(Marker, uint32_t);
            IMPLEMENT_GET_MAIN(DataSize, uint32_t);
            IMPLEMENT_GET_MAIN(CRC, uint32_t)

            IMPLEMENT_GET_AND_SET_MAIN(TrigNumLo, uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(TrigNumHi, uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(TrigNum, uint64_t)
            IMPLEMENT_GET_MAIN(NumMicroBlocks, uint16_t)
            IMPLEMENT_GET_AND_SET_MAIN(BuffId, uint16_t)
            IMPLEMENT_GET_AND_SET_MAIN(MonteCarloFlag, bool)
            IMPLEMENT_IS_MAIN(CRCCalculationUsed)
            IMPLEMENT_IS_AND_SET_MAIN(ConnectionInitialization)
            IMPLEMENT_IS_AND_SET_MAIN(CloseConnection)
            IMPLEMENT_IS_AND_SET_MAIN(MissingData)

            IMPLEMENT_FIGUREOUT_VERSION_STATIC(RawDataBlockHeader)

        protected:
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(setFloatingMicroBlockToBufferSource, const, bool, const void*)

            bool setFunctionPointers();

            /// These functions are not allowed to be executed as public
            bool addMicroBlock1(const void*     /*microblock_buffer*/,
                                const bool      /*check_mc_flag*/
                               ){return functionIsNotAllowed(__FUNCTION__, "Function is private.");}
            bool addMicroBlock2(const void*     /*microblock_buffer*/,
                                const uint32_t  /*microblock_size*/,
                                const bool      /*check_mc_flag*/
                               ){return functionIsNotAllowed(__FUNCTION__, "Function is private.");}
            bool addMicroBlock3(const void*     /*millislice_buffer*/,
                                const void*     /*microslice_buffer*/,
                                const version_t /*microblock_version*/,
                                const bool      /*check_mc_flag*/
                               ){return functionIsNotAllowed(__FUNCTION__, "Function is private.");}
            bool addMicroBlock4(const void*     /*millislice_buffer*/,
                                const void*     /*microslice_buffer*/,
                                const uint32_t  /*microslice_size*/,
                                const version_t /*microblock_version*/,
                                const bool      /*check_mc_flag*/
                               ){return functionIsNotAllowed(__FUNCTION__, "Function is private.");}

	    bool addDCMData1(const void*) { return functionIsNotAllowed(__FUNCTION__, "Function is private.");}
	    bool addDCMData2(const void*, const uint32_t) { return functionIsNotAllowed(__FUNCTION__,"Function is private.");}

//  allowed on a function-definition
    };// end of class definition
  
} // END NAMESPCE daqdataformats
#endif /* RAWDATABLOCK_H */
