#ifndef RAWEVENTV0_H
#define RAWEVENTV0_H
#include "DAQDataFormats/RawDAQData.h"
#include "DAQDataFormats/Macros.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawEvent
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE rawevent
#include "DAQDataFormats/DataFormatLast.inc"

// forward declarations
namespace daqdataformats{
    class RawEventHeader;
    class RawEventTail;
    class RawDataBlock;
    class RawTrigger;
}


namespace daqdataformats {

    namespace CURRENT_FIELDS_SUB_NAMESPACE{
        static const int32_t MC_FLAG_UNKNOWN = -1;

    }// end of namespace rawevent


namespace  VERSION_NAMESPACE {

   
  class RawEvent : public PREVIOUS_VERSION_CLASS_NAME {

        public:
            // Constructors
            RawEvent (const version_t); //! The default constructor
            RawEvent (const RawEvent&);
            /// Destructor
            ~RawEvent();

            void     init            ();
            using    PREVIOUS_VERSION_CLASS_NAME::print;
            void     print           (std::ostream& os=std::cout) const;
            uint32_t sizeofdata      () const;
            bool     setBufferSource(const void* pointer);
            using PREVIOUS_VERSION_CLASS_NAME::readData;	// unhide the reset of the readData methods
            void*                              readData(const void*);

        protected:

            ///Add a MicroSlice into the MilliSlice changing the MilliSlice Header appropriately
            /// If check_mc_flag = true, the method checks whether the DataBlock is MC generated
            /// and adds an appropriate flag to the Event Header.
            INIT_FIRST(addDataBlock        , , bool, const void*)
            ///Add a MicroSlice of into the MilliSlice changing the MilliSlice Header appropriately
            /// If check_mc_flag = true, the method checks whether the DataBlock is MC generated
            /// and adds an appropriate flag to the Event Header.
            INIT_FIRST(addDataBlockWithSize, , bool, const void*, const uint32_t)
            INIT_FIRST(close, , bool, const bool)

            INIT_FIRST(setTrigger1, , bool, const daqdataformats::RawTrigger&)
            INIT_FIRST(setTrigger2, , bool, const void*)

            INIT_FIRST(setFloatingDataBlockToBufferSource, const, bool, const void*)


            /// Next event. Advance the event number and clear the DataBlocks and CRC
	    INIT_FIRST(nextEvent           ,  , bool, )
            INIT_FIRST(setAppropriateMCFlag,  , bool, const void*)

	    INIT_FIRST(getHeader           , const, daqdataformats::RawEventHeader*, )
	    INIT_FIRST(getTail             , , daqdataformats::RawEventTail*, )
	    INIT_FIRST(getTrigger          , , daqdataformats::RawTrigger*, )
	    INIT_FIRST(getFloatingDataBlock, , daqdataformats::RawDataBlock*, )

	    INIT_FIRST(getHeaderConst           , const, daqdataformats::RawEventHeader*, )
	    INIT_FIRST(getTailConst             , const, daqdataformats::RawEventTail*, )
	    INIT_FIRST(getTriggerConst          , const, daqdataformats::RawTrigger*, )
	    INIT_FIRST(getFloatingDataBlockConst, const, daqdataformats::RawDataBlock*, )


	    INIT_FIRST(getHeaderMarkerLeft , const, uint32_t, )
	    INIT_FIRST(getHeaderMarkerRight, const, uint32_t, )
	    INIT_FIRST(getVersion          , const, uint32_t, )
	    INIT_FIRST(getDataLoggerID     , const, uint32_t, )
	    INIT_FIRST(getRunNumber        , const, uint32_t, )
	    INIT_FIRST(getRunType          , const, uint32_t, )
	    INIT_FIRST(getConfigIDX        , const, uint32_t, )
	    INIT_FIRST(getPartition        , const, uint32_t, )
	    INIT_FIRST(getSubrun           , const, uint32_t, )
	    INIT_FIRST(getEventNumberLo    , const, uint32_t, )
	    INIT_FIRST(getEventNumberHi    , const, uint32_t, )
	    INIT_FIRST(getEventNumber      , const, uint64_t, )
	    INIT_FIRST(getEventType        , const, uint32_t, )
	    INIT_FIRST(getEventSize        , const, uint32_t, )
	    INIT_FIRST(getDataBlockNumber  , const, uint32_t, )
	    INIT_FIRST(isEventIncomplete   , const, bool, )
	    INIT_FIRST(getMonteCarloFlag   , const, bool, )
	    INIT_FIRST(isCRCCalculationUsed, const, bool, )
            INIT_FIRST(setTriggerPointer   ,      , bool, const void*)

            INIT_FIRST(setDataLoggerID    , , bool, const uint32_t)
            INIT_FIRST(setRunNumber       , , bool, const uint32_t)
            INIT_FIRST(setRunType         , , bool, const uint32_t)
            INIT_FIRST(setConfigIDX       , , bool, const uint32_t)
            INIT_FIRST(setPartition       , , bool, const uint32_t)
            INIT_FIRST(setSubrun          , , bool, const uint32_t)
            INIT_FIRST(setEventNumberLo   , , bool, const uint32_t)
            INIT_FIRST(setEventNumberHi   , , bool, const uint32_t)
            INIT_FIRST(setEventNumber     , , bool, const uint64_t)
            INIT_FIRST(setEventType       , , bool, const uint32_t)
            INIT_FIRST(setEventIncomplete , , bool, const bool)
            INIT_FIRST(setMonteCarloFlag  , , bool, const bool)
            INIT_FIRST(setFloatingDataBlock, const ,bool, const uint32_t)

            uint32_t getPredefinedSize() const {return 1;}
        private:
            /// Attempt to set the trigger to the pointer
            bool tryToSetTrigger();
            /// Attempt to set the Event Tail to the pointer
            bool tryToSetTail   ();

        protected:
    
            daqdataformats::RawEventHeader*        _Header;                  //! RawEventV0 Header
            daqdataformats::RawEventTail*          _Tail;                    //! RawEventV0 Tail
            daqdataformats::RawTrigger*            _Trigger;                 //! Trigger
            mutable daqdataformats::RawDataBlock*  _FloatingDataBlock;       //! Floating DataBlock. The count starts from 0.
            mutable int32_t                        _FloatingDataBlockNumber; //! Floating DataBlock number

            version_t                              _triggerVersion;
            version_t                              _datablockVersion;

            int32_t                                _mcFlag;
            bool                                   _mcFlagBool;

            uint32_t                               _tailSize;
  };
  
  
}} // END NAMESPCE daqdataformats
#endif /* RAWEVENTV0_H */
