#ifndef DAQRAWTRIGGERTIMEMARKER_H
#define DAQRAWTRIGGERTIMEMARKER_H

#include "DAQDataFormats/RawTriggerTimingMarkerV0.h"
#include "DAQDataFormats/TriggerDefines.h"
#include "DAQDataFormats/FunctionBind.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawTriggerTimingMarker
#define  LAST_CLASS_VERSION           0
#define  CURRENT_FIELDS_SUB_NAMESPACE triggertimingmarker
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

    namespace triggertimingmarker {
        static const version_t RAWTRIGGERTIMIMGMARKER_LATEST_VERSION = LAST_CLASS_VERSION;
    }
  //! Class that contains the timing information relating to a given trigger
  /*!
    The RawTriggerTimingMarker contains the start indicator for data collection.

    The class supports general initialization with an internal or external
    data buffer as well as copyin style initialization for use with
    STL containers.
  */
  class RawTriggerTimingMarker: public PREVIOUS_VERSION_CLASS_NAME {

        public:

            RawTriggerTimingMarker(const version_t version = CURRENT_FIELDS_SUB_NAMESPACE::RAWTRIGGERTIMIMGMARKER_LATEST_VERSION);		                            //!< Constructor
            RawTriggerTimingMarker(const RawTriggerTimingMarker& copy_in);	    //!< Copyin Constructor
            RawTriggerTimingMarker(const void* buffer, const version_t version = rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION);  //!< Constructor with data intialization
            ~RawTriggerTimingMarker();                                    //!< Destructor

            IMPLEMENT_MAIN_STANDARD

            IMPLEMENT_GET_MAIN        (Version               , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(TriggerStartLow_baseClock , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(TriggerStartHigh_baseClock, uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(TriggerStartLow_833kHz    , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(TriggerStartHigh_833kHz   , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(TriggerStart_baseClock , uint64_t)
            IMPLEMENT_GET_AND_SET_MAIN(TriggerStart_833kHz    , uint64_t)

            IMPLEMENT_FIGUREOUT_VERSION_STATIC(CURRENT_CLASS)

       protected:
            IMPLEMENT_SET_MAIN(Version  , uint32_t)
            bool             setFunctionPointers();

            
  };
  
  
} // END NAMESPCE daqdataformats
#endif /* DAQRAWTRIGGERTIMEMARKER_H */
