#ifndef _DBUTILS_DAQCONFIG_DLRUNCONFIG_H
#define _DBUTILS_DAQCONFIG_DLRUNCONFIG_H

#include <DatabaseUtils/SimpleParameterSet.h>
#include <DatabaseUtils/DAQConfig/GlobalNameSpec.h>
#include <DatabaseUtils/DAQConfig/SubsystemNameSpec.h>
#include <boost/shared_ptr.hpp>
#include <iostream>
#include <map>
#include <string>
#include <vector>

namespace dbutils {
namespace daqconfig {

/**
 * Worker class to handle fetching of Data Logger run configuration data.
 *
 * @author Kurt Biery
 * @version $Revision: 1.4 $ $Date: 2012/02/25 17:11:44 $
 */

class DLRunConfig {

 public:

    typedef std::vector<boost::shared_ptr<SimpleParameterSet> > StreamList;

    // ********************
    // *** Constructors ***
    // ********************

    DLRunConfig(const std::vector<int64_t>& subsystemConfigIdList);

    DLRunConfig(const SubsystemNameSpec& nameSpec);

    DLRunConfig(const GlobalNameSpec& nameSpec);

    DLRunConfig(const int64_t& globalConfigId);

    virtual ~DLRunConfig() {};

    // *****************
    // *** Accessors ***
    // *****************

    virtual bool hasValidData() const {return _hasValidData;}
    int64_t getPrimarySubsystemConfigId() const {return _primarySubsystemConfigId;}

    StreamList getStreamList() {return _streamList;}
    std::string getOutputPath() {return _outputPath;}

 protected:

    bool _fetchAllData(const std::vector<int64_t>& subsystemConfigIdList);

    bool _fetchStreamData(const std::string& tableDefinitionFileName,
                          const int64_t& subsystemConfigId);

    bool _fetchSystemData(const std::string& tableDefinitionFileName,
                          const int64_t& subsystemConfigId);

    bool _hasValidData;
    int64_t _primarySubsystemConfigId;

    StreamList _streamList;
    std::string _outputPath;
};

};  // end of namespace daqconfig
};  // end of namespace dbutil

#endif
