#ifndef _RMS_MESSAGEASSEMBLER_H
#define _RMS_MESSAGEASSEMBLER_H

#include <rms/provider/MessageFragment.h>

#include <vector>
#include <string>

namespace gov {

namespace fnal {

namespace cd {

namespace rms {

namespace provider {

/**
 * This class is meant to collect and assemble a series
 * of message fragments into a complete message.
 *
 * @author Kurt Biery
 * @author Steve Foulkes
 * @version $Revision: 1.1.1.1 $ $Date: 2009/10/09 20:03:00 $
 */

class MessageAssembler {
 public:
    MessageAssembler(MessageFragment *initialFragment);
    ~MessageAssembler();
    bool addFragment(MessageFragment *fragment);
    bool hasCompleteMessage() const;
    
    std::string getMessage() const;

 private:
    /**
     * The UUID of the message that is being assembled.
     */
    char *_messageId;
    
    /**
     * A vector containing all the message fragments that have been
     * collected thus far.
     */
    std::vector <MessageFragment *> _fragments;
    
    /**
     * The total number of message fragments expected.
     */
    int _requiredFragmentCount;

};

};  // end of namespace provider

};  // end of namespace rms

};  // end of namespace cd

};  // end of namespace fnal

};  // end of namespace gov

#endif
