#include "DAQDataFormats/RawConfigurationHeader.h"
#include "DAQDataFormats/DataFormatException.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawConfigurationHeader
#define  LAST_CLASS_VERSION           1
#define  CURRENT_FIELDS_SUB_NAMESPACE rawconfhdr
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats{

//------------------------------------------------------------------------------
RawConfigurationHeader::RawConfigurationHeader(const version_t version):
    PREVIOUS_VERSION_CLASS_NAME(version)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    /// Perform general routines
    defaultConstructorFixedSize();
}

//------------------------------------------------------------------------------
RawConfigurationHeader::RawConfigurationHeader(const RawConfigurationHeader& copy_in):
    PREVIOUS_VERSION_CLASS_NAME(copy_in)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    lastOperationsInDeepCopy(copy_in);
}

//------------------------------------------------------------------------------
RawConfigurationHeader::RawConfigurationHeader(const void* datablockheader_buffer, const version_t version):
    PREVIOUS_VERSION_CLASS_NAME(version)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    copy(datablockheader_buffer);
}

//------------------------------------------------------------------------------
// Destructor
RawConfigurationHeader::~RawConfigurationHeader() { }


/// All binding routines
#define STANDARD_ROUTINES                                      \
            BIND_GET_AND_SET(ConfigurationHeaderVersion , version0, uint32_t);\
            BIND_GET_AND_SET(MarkerHi1 , version0, uint32_t);\
            BIND_GET_AND_SET(MarkerHi2 , version0, uint32_t);\
            BIND_GET_AND_SET(DetectorID , version0, uint32_t);\
            BIND_GET_AND_SET(ConfigurationBlockSize , version0, uint32_t);\
            BIND_GET_AND_SET(ResourceManagerID , version0, uint32_t);\
            BIND_GET_AND_SET(RunNumber , version0, uint32_t);\
            BIND_GET_AND_SET(RunType , version0, uint32_t);\
            BIND_GET_AND_SET(ConfigurationIDX , version0, uint32_t);\
            BIND_GET_AND_SET(Partition , version0, uint32_t);\
            BIND_GET_AND_SET(Subrun , version0, uint32_t);\
            BIND_GET_AND_SET(SystemNumber , version0, uint32_t);\
            BIND_GET_AND_SET(SystemVersionIDX , version0, uint32_t);\
            BIND_CUSTOM_FUNCTION(advanceSystemNumber, version0, , bool, ); \
            BIND_STADARD_FUNCTION(sizeofdata);                  \
            BIND_STADARD_FUNCTIONS;

#include "DAQDataFormats/setFunctionPointers_template.inc"





}// end of namespace
