#include "DAQDataFormats/RawDataBlockV2.h"
#include "DAQDataFormats/RawDataBlock.h"
#include "DAQDataFormats/DataFormatException.h"

#include <sys/time.h>		// gettimeofday
#include <stdarg.h>             // va_start
#include <stdio.h>              // printf

#include <boost/lexical_cast.hpp>
#include <boost/format.hpp>

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawDataBlock
#define  CURRENT_CLASS_VERSION        2
#define  CURRENT_FIELDS_SUB_NAMESPACE datablock
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats{
namespace VERSION_NAMESPACE{
//------------------------------------------------------------------------------
// Default constructor
  RawDataBlock::RawDataBlock(const version_t version, const uint32_t tlvl):
    PREVIOUS_VERSION_CLASS_NAME(version, tlvl)
    ,ZEROOUT_FUNCTION(addDCMData1)
    ,ZEROOUT_FUNCTION(addDCMData2)
{}
//------------------------------------------------------------------------------
RawDataBlock::RawDataBlock(const RawDataBlock& copyin):
     PREVIOUS_VERSION_CLASS_NAME(copyin)
    ,DEEPCOPY_FUNCTION(addDCMData1)
    ,DEEPCOPY_FUNCTION(addDCMData2)
{ }

//------------------------------------------------------------------------------
// Destructor
RawDataBlock::~RawDataBlock(){
}

//------------------------------------------------------------------------------
//Getters

//------------------------------------------------------------------------------
//Setters

//------------------------------------------------------------------------------
// Add a MicroBlock into the DataBlock
  bool RawDataBlock::addDCMData1(const void* dcmdata_buffer){

    PRINT_ON_DEBUG("In "<<__PRETTY_FUNCTION__<<"  0" )

      /// Construct a temporary nanoslice with external buffer
      const daqdataformats::RawSummaryDCMData tmp_dcmdata(dcmdata_buffer);

    PRINT_ON_DEBUG("In "<<__PRETTY_FUNCTION__<<"  1")
      PRINT_ON_DEBUG("About to add "<<tmp_dcmdata.sizeofdata()<<" words")

      RETURN_FUNCTION(addDCMData2)(dcmdata_buffer, tmp_dcmdata.sizeofdata());
  }

  //------------------------------------------------------------------------------
  // Add a Dcmdata of a given size into the DataBlock
  bool RawDataBlock::addDCMData2(const void* dcmdata_buffer, const uint32_t dcmdata_size){

    PRINT_ON_DEBUG("In "<<__PRETTY_FUNCTION__<<"  0")

      PRINT_ON_DEBUG("Size of internal buffer before reading "<<_InternalBuffer.size())


      /// Before reading, we need to adjust the size of the DataBlock
      /// Cannot rely on the size adjustment from readDataGeneral, because we don't read CRC from dcmdata
      const uint32_t new_sizeofdata = EXECUTE_FUNCTION(sizeofdata)() + dcmdata_size;

    /// If buffer is internal, we need to adjust its size
    if(_shouldBufferBeInternal) {
      /// Dcmdata size is the size of the header and the microslice

      /// Add the dcmdata size
      resizeInternalBuffer(new_sizeofdata);

      /// Check pointers after each read or resize buffer
      checkBufferInternalness();
    }// end of adjusting the internal buffer size
    
    /// Read the Dcmdata to the _Buffer starting from the previous position
    /// -1 because of CRC
    readDataGeneral(dcmdata_buffer, dcmdata_size, EXECUTE_FUNCTION(sizeofdata)() - 1);
    /// Check pointers after each read or resize buffer
    checkBufferInternalness();

    PRINT_ON_DEBUG("Size of internal buffer after reading "<<_InternalBuffer.size())

      /// Change the DataBlock header appropriately
      _Header->setDataSize(new_sizeofdata);
    _Header->advanceNumMicroBlocks();
    /// This should already have been set by addDroppedMicroblock...
    _Header->setMarker(daqdataformats::datablockheader::SummaryBlock_Marker);

    return true;
  }

  daqdataformats::RawSummaryDCMData* RawDataBlock::getDCMData()
  {
    //The DroppedMicroblock Summary should always be first, and it's 4 words long
    // The DCMData follows.
    void* pointer = (void*)&RAWBUFF32[_Header->sizeofdata() + 4];
    return new daqdataformats::RawSummaryDCMData(pointer , 0);
  }

}} // end of namespaces
