#include "DAQDataFormats/RawMicroSliceHeaderV0.h"
#include "DAQDataFormats/RawTimingMarker.h"
#include <boost/format.hpp>

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawMicroSliceHeader
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE rawmuhdr
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats{
namespace VERSION_NAMESPACE{

//------------------------Constructors------------------------------------------
RawMicroSliceHeader::RawMicroSliceHeader(const version_t version):
    PREVIOUS_VERSION_CLASS_NAME(version, (sizeofdata_t) &RawMicroSliceHeader::sizeofdata) ///< it is assumed that the size of data is the same
    ,ZEROOUT_GET_AND_SET       (Extra)
    ,ZEROOUT_GET_AND_SET       (Status)
{}

//------------------------------------------------------------------------------
RawMicroSliceHeader::RawMicroSliceHeader(const RawMicroSliceHeader& copyin):
     PREVIOUS_VERSION_CLASS_NAME(copyin)
    ,DEEPCOPY_GET_AND_SET       (Extra)
    ,DEEPCOPY_GET_AND_SET       (Status)
{}

//------------------------Destructor--------------------------------------------
RawMicroSliceHeader::~RawMicroSliceHeader() { }

//------------------------------------------------------------------------------
// Default initialization at wherever the pointer points
void RawMicroSliceHeader::init()
{
    initSoft();
    
    /// Set version in the buffer
    setVersion(_version);
}

//------------------------------------------------------------------------------
void RawMicroSliceHeader::initSoft(){
    for(uint32_t i=0; i< sizeofdata(); ++i){
        RAWBUFF32[i] = 0;
    }
    resetByteCount();
}

//------------------------------------------------------------------------------
// Add byte count. When adding a NanoSlice a ByteCount is increased by 12;
// After that. DataPresent is always true.
bool RawMicroSliceHeader::addByteCount(const uint32_t nbytes)
{
    setByteCount(getByteCount() + nbytes);
    return setDataPresent(true);
}

//------------------------------------------------------------------------------
bool RawMicroSliceHeader::resetByteCount()
{
    setByteCount(sizeof(uint32_t) * RAWTIMINGMARKER_SIZE);
    return setDataPresent(false);
}

//------------------------------------------------------------------------------
uint32_t RawMicroSliceHeader::getMicroSliceSize() const
{ 
    return sizeofdata() + getByteCount() / 4;
}

//------------------------------------------------------------------------------
uint32_t RawMicroSliceHeader::getStatus() const { GENERATE_FUNCTION_IS_NOT_ALLOWED; return 0;}
//------------------------------------------------------------------------------
bool     RawMicroSliceHeader::setStatus(const uint32_t) { return GENERATE_FUNCTION_IS_NOT_ALLOWED;}

//------------------------------------------------------------------------------
bool RawMicroSliceHeader::setVersion(const version_t val)
{
    uint32_t* pointer = &(RAWBUFF32[VERSION_NAMESPACE::CURRENT_FIELDS_SUB_NAMESPACE::WORD_Version]);
    BITFIELDSET(*pointer, val, VERSION_NAMESPACE::CURRENT_FIELDS_SUB_NAMESPACE::MASK_Version, VERSION_NAMESPACE::CURRENT_FIELDS_SUB_NAMESPACE::SHIFT_Version);
    return true;
}

//------------------------------------------------------------------------------
// Printing method
void RawMicroSliceHeader::print(std::ostream& os) const
{

    os<<boost::format("MicroSlice Header: Version=%d DP=%d SIM=%d Extra=%d Timeout=%d, LBC=%d\n")
	   % getVersion()
	   % getDataPresent()
	   % getMonteCarloFlag()
	   % EXECUTE_FUNCTION(getExtra)()
	   % getTimeout()
	   % getByteCount()
           ;

            printBuffer(os);

            //os<<"RAW(binary):";
            //printWord(0,true, os);
            //os<<std::endl;
}// end of print1

}}// end of daqdataformats::version0