/**
 * Ndmc implementation file
 * $Id: Ndmc.cpp,v 1.4 2011/05/11 17:46:08 qlu Exp $
 */
#include <sys/stat.h>
#include <string>
#include <iostream>
#include <boost/algorithm/string.hpp>
#include "messagefacility/MessageLogger/MessageLogger.h"
#include "NovaDAQMonitorClient/Ndmc.h"

using namespace NDMC_NAMESPACE;

/**
 * Static method to form full file path name for config file.
 *
 * @param configFile - input configuration file base name
 *                     If configFile begins with "/" then the configFile is assumed to
 *                     be absolute.
 *                     If configFile is empty, the default $SRT_PRIVATE(PUBLIC)_CONTEXT/
 *                     NovaDAQMonitorClient/config/NdmcConfiguration.xml is used.
 *                     If configFile is specified with a relative path, it will have
 *                     $SRT_PRIVATE(PUBLIC)_CONTEXT/NovaDAQMonitorClient/config/ prepended to it.
 * @return full file path name of configuration file
 */
std::string NDMC_NAMESPACE::getConfigFullFilePathName(std::string configFile) {

  std::string fullfilepathname;
  if ( configFile.find("/") == 0 ) {
    // User specified absolute file path
    fullfilepathname = configFile;
  }
  else {
    // User has specified a relative file name, prepend directory path
    struct stat st;
    if ( const char* envchar = getenv ("SRT_PRIVATE_CONTEXT") ) {
      fullfilepathname = std::string(envchar) + "/NovaDAQMonitorClient/config/";
    }
    if ( stat(fullfilepathname.c_str(),&st) != 0 ) {
      fullfilepathname = std::string(getenv("SRT_PUBLIC_CONTEXT"))+"/NovaDAQMonitorClient/config/";
    }

    if ( configFile.empty() ) {
       // Use default configuration
      fullfilepathname += "NdmcConfiguration.xml";
    }
    else {
      fullfilepathname += configFile;
    }
  }

  LOG_DEBUG("Ndmc") << "NDMC::getConfigFullFilePathName - return " <<
    fullfilepathname.c_str() << std::endl;

  return fullfilepathname;

}

