/**
 * NdmcClientService implementation file
 * $Id: NdmcClientService.cpp,v 1.7 2013/08/30 04:13:46 kasahara Exp $
 */
#include "messagefacility/MessageLogger/MessageLogger.h"
#include "NovaDAQMonitorClient/NdmcClientService.h"

using namespace NDMC_NAMESPACE;

std::string NdmcClientService::_CONFIGFILE = "/etc/ganglia/gmond_nova.conf";

/**
 * Constructor
 */
NdmcClientService::NdmcClientService() : _enabled(false) {

  LOG_DEBUG("NdmcClientService") << "NdmcClientService::ctor @" << this << std::endl;

  if ( _initGanglia() ) _enabled = true; // initialize Ganglia

}

/**
 * Destructor
 */
NdmcClientService::~NdmcClientService() {

  LOG_DEBUG("NdmcClientService") << "NdmcClientService::dtor @" << this << std::endl;

  Ganglia_pool_destroy(_globalContext);

}

/**
 * Access the singleton instance
 */
NdmcClientService& NdmcClientService::instance() {

  static NdmcClientService ndmc;
  return ndmc;

}

/**
 * Initialize Ganglia.  Return true if success, else false.
 */
bool NdmcClientService::_initGanglia() {
  
  // Create the Ganglia global context
  _globalContext = Ganglia_pool_create(NULL);
  if ( !_globalContext ) {
    ::mf::LogError("NdmcClientService") 
      << "NdmcClientService::_initGanglia - Unable to create global context." << std::endl;
    return false;
  }
  else {
    LOG_DEBUG("NdmcClientService") 
      << "NdmcClientService::_initGanglia - Successfully created global context." << std::endl;
  }

  // Parse the configuration file.  
  const char* configFile = getCONFIGFILE().c_str();
  ::mf::LogInfo("NdmcClientService") << "NdmcClientService::_initGanglia - Using gmond config "
                               << configFile << "." << std::endl;
  int fall_back_to_default = 0; // use default location for configuration, not.
  _gmondConfig = Ganglia_gmond_config_create((char*)configFile,fall_back_to_default);
  if ( !_gmondConfig ) {
    ::mf::LogError("NdmcClientService") << "NdmcClientService::_initGanglia - Unable to open "
				  << configFile << "." << std::endl;;
    return false;
  }
  else {
    LOG_DEBUG("NdmcClientService") 
              << "NdmcClientService::_initGanglia - Successfully opened "
              << configFile << "." << std::endl;
  }

  // Build the udp send channels
  _sendChannels = Ganglia_udp_send_channels_create(_globalContext, _gmondConfig);
  if ( !_sendChannels ) {
    ::mf::LogError("NdmcClientService") 
      << "NdmcClientService::_initGanglia - Unable to create ganglia send channels." << std::endl;
    return false;
  }
  else {
    LOG_DEBUG("NdmcClientService") 
      << "NdmcClientService::_initGanglia - Successfully created ganglia send channels." 
      << std::endl;;
  }

  return true;

}


