#include <NovaDAQUtilities/CachePolicy.h>

namespace novadaq {

/**
 * Tells the policy that a cache update was done.  This is used to
 * reset the state of the policy to "unexpired" after an update.
 */
void CachePolicy::cacheUpdated() {
    _markAsUpdated();
}

/**
 * Tests if the associated cache has expired based on the criteria
 * in the policy.
 *
 * @returns true if the cache has expired, false if not.
 */
bool CachePolicy::isExpired() {
    return _checkIfExpired();
}

/**
 * Tells the policy that a cache update was done.  This is used to
 * reset the state of the policy to "unexpired" after an update.
 */
void CachePolicy::_markAsUpdated() {
    // don't need to keep track of when the cache was updated since we
    // never expire in this simple implementation
}

/**
 * Tests if the associated cache has expired based on the criteria
 * in the policy.
 *
 * @returns true if the cache has expired, false if not.
 */
bool CachePolicy::_checkIfExpired() {
    return false;
}

};  // end of namespace novadaq
