#include <rms/util/ReentrantGetEnv.h>

namespace gov {

namespace fnal {

namespace cd {

namespace rms {

namespace util {

/**
 * getenv() isn't threadsafe, so use a mutex
 * to protect it and copy the output to a 
 * string.
 *
 * @param name The environment variable to retrieve
 * @return The value of the requested environment variable
 * if it exists, an empty string otherwise.
 */
std::string ReentrantGetEnv(const std::string name) {
    char *envPtr;
    static boost::mutex getEnvLock;
    
    boost::mutex::scoped_lock lk(getEnvLock);

    envPtr = getenv(name.c_str());

    if (envPtr) {
        return std::string(envPtr);
    }
    else {
        return std::string("");
    }
}

};  // end of namespace util

};  // end of namespace rms

};  // end of namespace cd

};  // end of namespace fnal

};  // end of namespace gov
